﻿using System;
using System.Linq;
using System.Collections.Generic;

// このクラスは、追加、編集、削除、フェッチなどのアクションに基づいてスケジュールを設定します。
public static class Calendar {

    public static void UpdateCalendarColor(int userId, int calendarId, string color) {
        var db = UserHelper.DatabaseInstance;
        db.Execute("UPDATE CalendarsUsers SET Color = @0 WHERE CalendarId = @1 AND UserId = @2", color, calendarId, userId);
    }

    public static void UpdateCalendar(int userId, int calendarId, string calendarName = null, string color = null) {
        var db = UserHelper.DatabaseInstance;

        color = color ?? ColorHelper.GetRandomColor();
        calendarName = calendarName ?? "Default";

        // カレンダーを編集しています
        db.Execute("UPDATE Calendars SET Name = @0 WHERE Id = @1", calendarName, calendarId);
        UpdateCalendarColor(userId, calendarId, color);
    }

    public static int CreateCalendar(int userId, string calendarName = null, string color = null) {
        var db = UserHelper.DatabaseInstance;

        color = color ?? ColorHelper.GetRandomColor();
        calendarName = calendarName ?? "Default";

        // 新しいカレンダーを作成します
        db.Execute("INSERT INTO Calendars (Name, Creator, Guid) Values (@0, @1, @2)", calendarName, userId, GenerateUniqueId());

        int calendarId = Convert.ToInt32(db.GetLastInsertId());

        // カレンダー ID を CalendarUsers に追加します
        db.Execute("INSERT INTO CalendarsUsers (CalendarId, UserId, Permissions, Color) VALUES (@0, @1, @2, @3)", calendarId, userId, Permission.Own, color);

        return calendarId;
    }

    private static string GenerateUniqueId() {
        return Guid.NewGuid().ToString();

    }

    /// <summary>
    /// カレンダーとそのカレンダー上のすべてのイベントを削除します
    /// </summary>
    /// <param name="calendarId">削除するカレンダーの ID</param>
    public static void DeleteCalendar(int calendarId) {
        var db = UserHelper.DatabaseInstance;

        db.Execute("DELETE FROM Events WHERE CalendarId = @0", calendarId);
        db.Execute("DELETE FROM CalendarsUsers WHERE CalendarId = @0", calendarId);
        db.Execute("DELETE FROM Calendars WHERE Id = @0", calendarId);
    }

    /// <summary>
    /// userId がアクセス許可を持っているカレンダーを取得します。
    /// </summary>
    /// <param name="userId">検索するユーザーの ID</param>
    /// <param name="minimumPermission">カレンダーに必要なユーザーのアクセス許可の最小値</param>
    /// <returns>アクセス許可 (降順) で並べ替えてから名前 (昇順) で並べ替えられた行のセット</returns>
    public static dynamic GetUserCalendars(int userId, Permission minimumPermission = Permission.View) {
        var db = UserHelper.DatabaseInstance;

        return db.Query(@"SELECT c.Name, c.Creator, c.Guid, cu.* 
                            FROM Calendars AS c 
                            JOIN CalendarsUsers AS cu ON c.Id = cu.CalendarId
                            WHERE cu.UserId = @0
                            AND cu.Permissions >= @1
                            ORDER BY cu.Permissions DESC, c.name ASC",
                            userId, minimumPermission);
    }

    /// <summary>
    /// ユーザーとカレンダーの組み合わせで指定されたカレンダー情報を取得します
    /// </summary>
    /// <param name="userId">ユーザーの ID</param>
    /// <param name="calendarId">カレンダーの ID</param>
    /// <returns>カレンダーが存在しない場合は null、それ以外の場合はカレンダー情報を返します</returns>
    public static dynamic GetUserCalendar(int userId, int calendarId) {
        var db = UserHelper.DatabaseInstance;

        return db.QuerySingle(@"SELECT c.Name, c.Creator, c.Guid, cu.*
                                FROM Calendars AS c
                                JOIN CalendarsUsers AS cu ON c.Id = cu.CalendarId
                                WHERE cu.UserId = @0
                                AND cu.CalendarId = @1",
                                userId, calendarId);
    }

    public static dynamic GetCalendarGroups(int userId) {
        // ページに読み込まれたすべてのカレンダーのキャッシュを保持して
        // カレンダーのクエリと結合を減らす必要があります
        var calendars = Calendar.GetUserCalendars(userId);

        var calendarGroups =
            (from dynamic c in (IEnumerable<object>)calendars
             group c by c.Permissions >= (int)Permission.Own into g
             orderby g.Key
             select new {
                 Own = g.Key ? "Mine" : "Shared",
                 Calendars = g.OrderBy(c => c.Name)
             }).ToDictionary(g => g.Own, g => g.Calendars);

        return calendarGroups;
    }

    public static dynamic GetCalendar(int calendarId) {
        var db = UserHelper.DatabaseInstance;
        return db.QuerySingle("SELECT * FROM Calendars WHERE Id = @0", calendarId);
    }

    public static dynamic GetCalendarByHash(int calendarId, string hash) {
        var db = UserHelper.DatabaseInstance;
        return db.QuerySingle("SELECT * FROM Calendars WHERE Id = @0 AND Guid = @1", calendarId, hash);
    }
}