﻿using System;

public static class Event {
    /// <summary>
    /// カレンダーにイベントを追加します
    /// </summary>
    /// <param name="userId">イベントの作成者の ID</param>
    /// <param name="calendarId">イベントを追加するカレンダーの ID</param>
    /// <param name="name">イベントの名前</param>
    /// <param name="description">イベントの説明</param>
    /// <param name="location">イベントの場所</param>
    /// <param name="utcStart">イベントの開始</param>
    /// <param name="utcEnd">イベントの終了</param>
    /// <param name="allDay">イベントが終日かどうかを指定します。イベントが終日の場合は、
    /// 開始と終了の TIME はロールを実行しません</param>
    /// <remarks>同じ日に開始し、終了するイベントのみがサポートされています</remarks>
    /// <returns>追加されるイベントの ID</returns>
    public static int AddEvent(int userId, int calendarId, string name, string description, string location, DateTime utcStart, DateTime utcEnd, bool allDay = false) {
        var db = UserHelper.DatabaseInstance;
        db.Query(@"INSERT INTO Events
                    (OrganizerId,   CalendarId,     Name,   Description,    Location,   AllDay, Start,  [End])
             Values (@0,            @1,             @2,             @3,         @4,     @5,     @6,     @7)",
                    userId, calendarId, name, description, location, allDay, utcStart, utcEnd);

        return Convert.ToInt32(db.GetLastInsertId());
    }

    /// <summary>
    /// イベントを編集します
    /// </summary>
    /// <see>Event.AddEvent</see>>
    /// <param name="eventId">編集するイベントの ID</param>
    public static void EditEvent(int eventId, int calendarId, string name, string description, string location, DateTime utcStart, DateTime utcEnd, bool allDay = false) {
        var db = UserHelper.DatabaseInstance;

        db.Execute(@"
                    UPDATE Events
                    SET CalendarId = @0, Name = @1, Description = @2, Location = @3, AllDay = @4, Start = @5, [End] = @6
                    WHERE Id = @7",
                    calendarId, name, description, location, allDay, utcStart, utcEnd,
                    eventId);
    }

    /// <summary>
    /// イベントを削除します
    /// </summary>
    /// <param name="eventId">削除するイベントの ID</param>
    public static void DeleteEvent(int eventId) {
        var db = UserHelper.DatabaseInstance;

        db.Execute("DELETE FROM EVENTS WHERE Id = @0", eventId);
    }

    /// <summary>
    /// ユーザー-カレンダー-イベントの組み合わせに関する情報を取得します
    /// </summary>
    /// <param name="userId">検索するユーザー ID</param>
    /// <param name="eventId">検索するイベント ID</param>
    /// <returns>結果がその組み合わせと一致しない場合は Null、それ以外の場合は組み合わせに関する情報</returns>
    public static dynamic GetUserEvent(int userId, int eventId) {
        var db = UserHelper.DatabaseInstance;
        return db.QuerySingle(@"
            SELECT e.*, c.Name AS CalendarName, c.Id AS CalendarId, cu.Color, cu.Permissions
            FROM EVENTS AS e 
            JOIN Calendars AS c ON e.CalendarID = c.Id 
            JOIN CalendarsUsers AS cu ON c.Id = cu.CalendarId
            WHERE cu.UserId = @0 AND e.Id = @1",
            userId, eventId);
    }

    public static dynamic GetCalendarEvents(int calendarId) {
        var db = UserHelper.DatabaseInstance;
        return db.Query("SELECT * FROM Events WHERE CalendarId = @0", calendarId);
    }
}